@@ -7,6 +7,9 @@ from inspect import isclass
 import re
 
 from peewee import *
+from peewee import _StringField
+from peewee import _query_val_transform
+from peewee import text_type
 try:
     from pymysql.constants import FIELD_TYPE
 except ImportError:
@@ -39,7 +42,7 @@ class Column(object):
 
     def __init__(self, name, field_class, raw_column_type, nullable,
                  primary_key=False, column_name=None, index=False,
-                 unique=False, extra_parameters=None):
+                 unique=False, default=None, extra_parameters=None):
         self.name = name
         self.field_class = field_class
         self.raw_column_type = raw_column_type
@@ -48,6 +51,7 @@ class Column(object):
         self.column_name = column_name
         self.index = index
         self.unique = unique
+        self.default = default
         self.extra_parameters = extra_parameters
 
         # Foreign key metadata.
@@ -77,8 +81,10 @@ class Column(object):
             params['null'] = True
         if self.field_class is ForeignKeyField or self.name != self.column_name:
             params['column_name'] = "'%s'" % self.column_name
-        if self.primary_key and self.field_class is not AutoField:
+        if self.primary_key and not issubclass(self.field_class, AutoField):
             params['primary_key'] = True
+        if self.default is not None:
+            params['constraints'] = '[SQL("DEFAULT %s")]' % self.default
 
         # Handle ForeignKeyField-specific attributes.
         if self.is_foreign_key():
@@ -164,16 +170,22 @@ class Metadata(object):
             pk = pk_names[0]
             if column_types[pk] is IntegerField:
                 column_types[pk] = AutoField
+            elif column_types[pk] is BigIntegerField:
+                column_types[pk] = BigAutoField
 
         columns = OrderedDict()
         for name, column_data in metadata.items():
+            field_class = column_types[name]
+            default = self._clean_default(field_class, column_data.default)
+
             columns[name] = Column(
                 name,
-                field_class=column_types[name],
+                field_class=field_class,
                 raw_column_type=column_data.data_type,
                 nullable=column_data.null,
                 primary_key=column_data.primary_key,
                 column_name=name,
+                default=default,
                 extra_parameters=extra_params.get(name))
 
         return columns
@@ -181,6 +193,15 @@ class Metadata(object):
     def get_column_types(self, table, schema=None):
         raise NotImplementedError
 
+    def _clean_default(self, field_class, default):
+        if default is None or field_class in (AutoField, BigAutoField) or \
+           default.lower() == 'null':
+            return
+        if issubclass(field_class, _StringField) and \
+           isinstance(default, text_type) and not default.startswith("'"):
+            default = "'%s'" % default
+        return default or "''"
+
     def get_foreign_keys(self, table, schema=None):
         return self.database.get_foreign_keys(table, schema)
 
@@ -347,16 +368,19 @@ class SqliteMetadata(Metadata):
         'date': DateField,
         'datetime': DateTimeField,
         'decimal': DecimalField,
+        'float': FloatField,
         'integer': IntegerField,
         'integer unsigned': IntegerField,
         'int': IntegerField,
         'long': BigIntegerField,
+        'numeric': DecimalField,
         'real': FloatField,
         'smallinteger': IntegerField,
         'smallint': IntegerField,
         'smallint unsigned': IntegerField,
         'text': TextField,
         'time': TimeField,
+        'varchar': CharField,
     }
 
     begin = '(?:["\[\(]+)?'
@@ -432,8 +456,10 @@ class Introspector(object):
             metadata = PostgresqlMetadata(database)
         elif isinstance(database, MySQLDatabase):
             metadata = MySQLMetadata(database)
-        else:
+        elif isinstance(database, SqliteDatabase):
             metadata = SqliteMetadata(database)
+        else:
+            raise ValueError('Introspection not supported for %r' % database)
         return cls(metadata, schema=schema)
 
     def get_database_class(self):
@@ -457,8 +483,14 @@ class Introspector(object):
             model_name = 'T' + model_name
         return model_name
 
-    def make_column_name(self, column):
-        column = re.sub('_id$', '', column.lower().strip()) or column.lower()
+    def make_column_name(self, column, is_foreign_key=False):
+        column = column.lower().strip()
+        if is_foreign_key:
+            # Strip "_id" from foreign keys, unless the foreign-key happens to
+            # be named "_id", in which case the name is retained.
+            column = re.sub('_id$', '', column) or column
+
+        # Remove characters that are invalid for Python identifiers.
         column = re.sub('[^\w]+', '_', column)
         if column in RESERVED_WORDS:
             column += '_'
@@ -466,15 +498,17 @@ class Introspector(object):
             column = '_' + column
         return column
 
-    def introspect(self, table_names=None, literal_column_names=False):
+    def introspect(self, table_names=None, literal_column_names=False,
+                   include_views=False):
         # Retrieve all the tables in the database.
-        if self.schema:
-            tables = self.metadata.database.get_tables(schema=self.schema)
-        else:
-            tables = self.metadata.database.get_tables()
+        tables = self.metadata.database.get_tables(schema=self.schema)
+        if include_views:
+            views = self.metadata.database.get_views(schema=self.schema)
+            tables.extend([view.name for view in views])
 
         if table_names is not None:
             tables = [table for table in tables if table in table_names]
+        table_set = set(tables)
 
         # Store a mapping of table name -> dictionary of columns.
         columns = {}
@@ -501,17 +535,28 @@ class Introspector(object):
             except ValueError as exc:
                 err(*exc.args)
                 foreign_keys[table] = []
+            else:
+                # If there is a possibility we could exclude a dependent table,
+                # ensure that we introspect it so FKs will work.
+                if table_names is not None:
+                    for foreign_key in foreign_keys[table]:
+                        if foreign_key.dest_table not in table_set:
+                            tables.append(foreign_key.dest_table)
+                            table_set.add(foreign_key.dest_table)
 
             model_names[table] = self.make_model_name(table)
 
+            # Collect sets of all the column names as well as all the
+            # foreign-key column names.
             lower_col_names = set(column_name.lower()
                                   for column_name in table_columns)
+            fks = set(fk_col.column for fk_col in foreign_keys[table])
 
             for col_name, column in table_columns.items():
                 if literal_column_names:
                     new_name = re.sub('[^\w]+', '_', col_name)
                 else:
-                    new_name = self.make_column_name(col_name)
+                    new_name = self.make_column_name(col_name, col_name in fks)
 
                 # If we have two columns, "parent" and "parent_id", ensure
                 # that when we don't introduce naming conflicts.
@@ -578,9 +623,10 @@ class Introspector(object):
             indexes)
 
     def generate_models(self, skip_invalid=False, table_names=None,
-                        literal_column_names=False, bare_fields=False):
-        database = self.introspect(table_names=table_names,
-                                   literal_column_names=literal_column_names)
+                        literal_column_names=False, bare_fields=False,
+                        include_views=False):
+        database = self.introspect(table_names, literal_column_names,
+                                   include_views)
         models = {}
 
         class BaseModel(Model):
@@ -646,7 +692,13 @@ class Introspector(object):
 
                     # Generate a unique related name.
                     params['backref'] = '%s_%s_rel' % (table, column_name)
-                if column_name in column_indexes and not column.is_primary_key():
+
+                if column.default is not None:
+                    constraint = SQL('DEFAULT %s' % column.default)
+                    params['constraints'] = [constraint]
+
+                if column_name in column_indexes and not \
+                   column.is_primary_key():
                     if column_indexes[column_name]:
                         params['unique'] = True
                     elif not column.is_foreign_key():
@@ -671,3 +723,26 @@ class Introspector(object):
 def introspect(database, schema=None):
     introspector = Introspector.from_database(database, schema=schema)
     return introspector.introspect()
+
+
+def generate_models(database, schema=None, **options):
+    introspector = Introspector.from_database(database, schema=schema)
+    return introspector.generate_models(**options)
+
+
+def print_model(model):
+    print(model._meta.name)
+    for field in model._meta.sorted_fields:
+        print('  %s %s' % (field.name, field.field_type))
+
+
+def get_table_sql(model):
+    sql, params = model._schema._create_table().query()
+    if not params:
+        return sql
+    elif model._meta.database.param != '%s':
+        sql = sql.replace(model._meta.database.param, '%s')
+    return sql % tuple(map(_query_val_transform, params))
+
+def print_table_sql(model):
+    print(get_table_sql(model))
